/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/


// PicoloVideoTriggerDoc.cpp : implementation of the CPicoloVideoTriggerDoc class
//
#include "stdafx.h"
#include "PicoloVideoTrigger.h"

#include "PicoloVideoTriggerDoc.h"
#include "PicoloVideoTriggerView.h"


#include <stdlib.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPicoloVideoTriggerDoc

IMPLEMENT_DYNCREATE(CPicoloVideoTriggerDoc, CDocument)

BEGIN_MESSAGE_MAP(CPicoloVideoTriggerDoc, CDocument)
    //{{AFX_MSG_MAP(CPicoloVideoTriggerDoc)
    ON_COMMAND(ID_GO, OnGo)
    ON_COMMAND(ID_STOP, OnStop)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPicoloVideoTriggerDoc construction/destruction

CPicoloVideoTriggerDoc::CPicoloVideoTriggerDoc()
{
    m_Channel = 0;
    m_pCurrent = NULL;
    m_SizeX = 0;
    m_SizeY = 0;
    m_BufferPitch = m_SizeX;
    m_bScreenRefreshCompleted = TRUE;
}

CPicoloVideoTriggerDoc::~CPicoloVideoTriggerDoc()
{
    // Set the channel to IDLE before deleting it
    McSetParamInt(m_Channel, MC_ChannelState, MC_ChannelState_IDLE);

    // Delete the channel
    McDelete(m_Channel);
}

BOOL CPicoloVideoTriggerDoc::OnNewDocument()
{
    if (!CDocument::OnNewDocument())
        return FALSE;

    // + PicoloVideoTrigger Sample Program

    // Show scope of the sample program
    MessageBox(NULL, 
        "This program demonstrates the Video Acquisition Mode on a Picolo Board.\n"
        "\n"
        "The \"Go!\" menu starts an acquisition sequence of 100 frames by triggering the channel.\n"
        "The sequence can also be triggered by a GOLOW transition of a hardware trigger signal on the IN1 input.\n"
        "By default, this program requires a PAL camera connected on VID1.",
            "MultiCam sample description", MB_OK);


    // Create a channel and associate it with the first connector on the first board
    McCreate(MC_CHANNEL, &m_Channel);
    McSetParamInt(m_Channel, MC_DriverIndex, 0);
    McSetParamStr(m_Channel, MC_Connector, "VID1");
    // Choose the video standard
    McSetParamStr(m_Channel, MC_CamFile, "PAL");
    // Choose the pixel color format
    McSetParamInt(m_Channel, MC_ColorFormat, MC_ColorFormat_RGB24);

    // Choose the acquisition mode
    McSetParamInt(m_Channel, MC_AcquisitionMode, MC_AcquisitionMode_VIDEO);
    // Choose the way the first acquisition is triggered
    McSetParamInt(m_Channel, MC_TrigMode, MC_TrigMode_COMBINED);
    // Choose the triggering mode for subsequent acquisitions
    McSetParamInt(m_Channel, MC_NextTrigMode, MC_NextTrigMode_REPEAT);
    // Choose the number of sequences
    McSetParamInt(m_Channel, MC_ActivityLength, MC_INDETERMINATE);
    // Choose the number of images to acquire
    McSetParamInt(m_Channel, MC_SeqLength_Fr, 100);
    McSetParamInt(m_Channel, MC_TrigLineIndex, 1);
    McSetParamInt(m_Channel, MC_TrigEdge, MC_TrigEdge_GOLOW);


    // Retrieve image dimensions
    McGetParamInt(m_Channel, MC_ImageSizeX, &m_SizeX);
    McGetParamInt(m_Channel, MC_ImageSizeY, &m_SizeY);
    McGetParamInt(m_Channel, MC_BufferPitch, &m_BufferPitch);

    // The memory allocation for the images is automatically done by Multicam when activating the channel.
    // We only set the number of surfaces to be created by MultiCam.
    McSetParamInt(m_Channel, MC_SurfaceCount, EURESYS_SURFACE_COUNT);

    // Enable MultiCam signals
    McSetParamInt(m_Channel, MC_SignalEnable + MC_SIG_SURFACE_PROCESSING, MC_SignalEnable_ON);
    McSetParamInt(m_Channel, MC_SignalEnable + MC_SIG_ACQUISITION_FAILURE, MC_SignalEnable_ON);

    // Register the callback function
    McRegisterCallback(m_Channel, GlobalCallback, this);

    // - PicoloVideoTrigger Sample Program
    
    return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// GlobalCallback

void WINAPI GlobalCallback(PMCSIGNALINFO SigInfo)
{
    if (SigInfo && SigInfo->Context)
    {
        CPicoloVideoTriggerDoc* pDoc = (CPicoloVideoTriggerDoc*) SigInfo->Context;
        pDoc->Callback (SigInfo);
    }
}

/////////////////////////////////////////////////////////////////////////////
// Callback

void CPicoloVideoTriggerDoc::Callback(PMCSIGNALINFO SigInfo)
{
    // + PicoloVideoTrigger Sample Program

    switch(SigInfo->Signal)
    {
        case MC_SIG_SURFACE_PROCESSING:
            // Update "current" surface address pointer
            McGetParamInt (SigInfo->SignalInfo, MC_SurfaceAddr, (PINT32) &m_pCurrent); 

            //----------------------------------------
            //
            // Insert the eVision code here.
            //
            //----------------------------------------

            // Post screen refresh message only if previous refresh completed
            if(m_bScreenRefreshCompleted)
            {
                m_bScreenRefreshCompleted = FALSE;
                POSITION pos = GetFirstViewPosition();
                CPicoloVideoTriggerView *pView = (CPicoloVideoTriggerView *)GetNextView(pos);
                if (pView == NULL) return;

                HWND hWindow = pView->GetSafeHwnd();
                if (hWindow == NULL) return;

                RECT recpict ;
                recpict.left =0;
                recpict.top =0;
                recpict.right = m_SizeX - 1;
                recpict.bottom = m_SizeY - 1;
                InvalidateRect(hWindow,&recpict,FALSE);
            }
            break;
        case MC_SIG_ACQUISITION_FAILURE:
            MessageBox(NULL, "Acquisition Failure !", "PicoloVideoTrigger", MB_OK);
            break;
        default:
            break;

    }
    // - PicoloVideoTrigger Sample Program
}



/////////////////////////////////////////////////////////////////////////////
// CPicoloVideoTriggerDoc commands

void CPicoloVideoTriggerDoc::OnGo()
{
    // + PicoloVideoTrigger Sample Program
    
    // Start an acquisition sequence by activating the channel
    McSetParamInt(m_Channel, MC_ChannelState, MC_ChannelState_ACTIVE);

    // Generate a soft trigger event (STRG)
    McSetParamInt(m_Channel, MC_ForceTrig, MC_ForceTrig_TRIG);
    
    // - PicoloVideoTrigger Sample Program
}

void CPicoloVideoTriggerDoc::OnStop()
{
    // + PicoloVideoTrigger Sample Program
    
    // Stop an acquisition sequence by deactivating the channel
    McSetParamInt(m_Channel, MC_ChannelState, MC_ChannelState_IDLE);
    
    // - PicoloVideoTrigger Sample Program
    
    UpdateAllViews(NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CPicoloVideoTriggerDoc serialization

void CPicoloVideoTriggerDoc::Serialize(CArchive& ar)
{
}

/////////////////////////////////////////////////////////////////////////////
// CPicoloVideoTriggerDoc diagnostics

#ifdef _DEBUG
void CPicoloVideoTriggerDoc::AssertValid() const
{
    CDocument::AssertValid();
}

void CPicoloVideoTriggerDoc::Dump(CDumpContext& dc) const
{
    CDocument::Dump(dc);
}
#endif //_DEBUG
